package com.samsung.android.sdk.professionalaudio.sample.simpleclient;

import android.Manifest;
import android.annotation.TargetApi;
import android.app.Activity;
import android.content.pm.PackageManager;
import android.os.Build;
import android.os.Bundle;
import android.util.AndroidRuntimeException;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;

import com.samsung.android.sdk.SsdkUnsupportedException;
import com.samsung.android.sdk.professionalaudio.Sapa;
import com.samsung.android.sdk.professionalaudio.SapaProcessor;
import com.samsung.android.sdk.professionalaudio.SapaService;

public class SapaSimpleClientActivity extends Activity {

    private final static String TAG = "SapaSimpleClient";

    Button mPlayButton, mActivateButton, mDeactivateButton;

    SapaService mService;
    SapaProcessor mClient;
    private static final int REQUEST_CODE_LOCATION = 204;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_apasimple_client);

        int sdkVersion = Build.VERSION.SDK_INT;
        if (sdkVersion >= Build.VERSION_CODES.M) {
            checkPermission();
        } else {
            startSapaService();
        }

        mPlayButton = (Button) this.findViewById(R.id.play_button);
        mPlayButton.setOnClickListener(new Button.OnClickListener() {
            public void onClick(View v) {
                if (mClient != null) {
                    mClient.activate();
                    mPlayButton.setText(R.string.playing_sound);
                    mPlayButton.setEnabled(false);
                }
            }
        });

        mActivateButton = (Button) this.findViewById(R.id.button_activate);
        mActivateButton.setOnClickListener(new Button.OnClickListener() {
            public void onClick(View v) {
                if (mClient != null)
                    mClient.activate();
            }
        });

        mDeactivateButton = (Button) this.findViewById(R.id.button_deactivate);
        mDeactivateButton.setOnClickListener(new Button.OnClickListener() {
            public void onClick(View v) {
                if (mClient != null)
                    mClient.deactivate();
            }
        });
    }

    private boolean startSapaService() {
        try {
            Sapa sapa = new Sapa();
            sapa.initialize(this);
            mService = new SapaService();
            mService.stop(true);
            mService.start(SapaService.START_PARAM_DEFAULT_LATENCY);
            mClient = new SapaProcessor(this, null, new SapaProcessor.StatusListener() {

                @Override
                public void onKilled() {
                    Log.v(TAG, "SapaSimpleClient will be closed. because of the SapaProcessor was closed.");
                    mService.stop(true);
                    finish();
                }
            });
            mService.register(mClient, false);

        } catch (SsdkUnsupportedException e) {
            e.printStackTrace();
            Toast.makeText(this, "not support professional audio",
                    Toast.LENGTH_LONG).show();
            finish();
            return false;
        } catch (InstantiationException e) {
            e.printStackTrace();
            Toast.makeText(this, "fail to instantiate", Toast.LENGTH_LONG)
                    .show();
            finish();
            return false;
        } catch (AndroidRuntimeException e) {
            e.printStackTrace();
            Toast.makeText(this, "fail to start", Toast.LENGTH_LONG)
                    .show();
            finish();
            return false;
        }
        return true;
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (mService != null) {
            if (mClient != null) {
                mService.unregister(mClient);
            }
            mService.stop(true);
        }
    }

    @TargetApi(Build.VERSION_CODES.M)
    private void checkPermission() {
        if (checkSelfPermission(Manifest.permission.RECORD_AUDIO) != PackageManager.PERMISSION_GRANTED) {
            requestPermissions(new String[]{
                            Manifest.permission.RECORD_AUDIO
                    },
                    REQUEST_CODE_LOCATION);
        } else {
            startSapaService();
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String[] permissions, int[] grantResults) {

        switch (requestCode) {
            case REQUEST_CODE_LOCATION:
                if (grantResults.length > 0 && grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                    startSapaService();
                } else {
                    finish();
                }
                break;
            default:
                break;
        }
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
    }
}
