/*
 * Copyright (c) 2007 Samsung Electronics Co., Ltd. All rights reserved. 
 */

#ifndef ANDROID_APA_INTERFACEV2_H
#define ANDROID_APA_INTERFACEV2_H

#include <stdint.h>
#include <IAPAInterface.h>

namespace android {

/**
 * \brief An interface version 2 for a APA Client
 * 
 * This is the entry interface for processing module.@n
 * All of processing module has an APA Client.@n
 * You should implement this.@n
 * 
 */
class IAPAInterfaceV2 : public IAPAInterface {
	public:
		/**
		 * Constructor
		 */
		IAPAInterfaceV2();
		
		
		/**
		 * Destructor
		 */
		virtual ~IAPAInterfaceV2();
		
		/**
		 * Handle the stream data from the Java side
		 * @param stream a byte array 
		 * @param size stream's array size
		 * @param id the identification number.
		 * @return APA_RETURN_SUCCESS, APA_RETURN_ERROR
		 */
		virtual int sendStream(const void* stream, const long size, const int id) = 0;
		
		/**
		 * Return the Target device API Level.
		 */
		int getTargetAPILevel();
};

};

#endif // ANDROID_APA_INTERFACEV2_H